--[[
************************************************************************
Project				: SKG
Author				: zhinjio
Project Revision	: r127
Project Date		: 20090125215842

File				: SKG.lua
Commit Author		: zhinjio
Commit Revision		: 126
Commit Date			: 20090125215023
************************************************************************
Description	:
	Main addon primary methods (oninit, onenable, ondisable) and
	associated goo.

TODO		:
	Not sure why my timer frame code is in here. Is there maybe a better
	place for it?
************************************************************************
(see bottom of file for changelog)
************************************************************************
--]]

local MODNAME = "SKG"
local FULLNAME = MODNAME
local SKG = LibStub( "AceAddon-3.0" ):GetAddon( MODNAME, "AceEvent-3.0" )
local L = LibStub:GetLibrary( "AceLocale-3.0" ):GetLocale( MODNAME )
local AceConfig 		= LibStub("AceConfig-3.0")
local AceConfigReg 		= LibStub("AceConfigRegistry-3.0")
local AceConfigDialog 	= LibStub("AceConfigDialog-3.0")

local addon = SKG

local addonversion = "r127"
local addonauthor = "zhinjio"
local builddate = "20090125215842"

local modularOptions = {}

-- addon globals
-- addon.someValue = 0
addon.me = ""

local db

-- Data Broker objects and functions
local LDB = LibStub( "LibDataBroker-1.1" )
local launcher = LDB:NewDataObject( MODNAME, {
	type = "data source",
	text = "",
	label = FULLNAME,
	icon = "Interface\\Addons\\" .. FULLNAME .. "\\icon",
	-- Right click gets you config, left click toggles main frame
	OnClick = function( clickedframe, button )
				if button == "RightButton" then
					addon:ShowConfig()
				else
					addon:Toggle()
				end
	end,
} )

-- keep out database in tip top shape
function addon.upgradeDB()
	local olddb = addon.db.profile
	if not olddb.Options.General.dbVersion then
		-- file existed before I added in db versioning. Do the right thing.
		olddb.Options.General.dbVersion = "1.0"
		addon.Print( string.format( L["Database upgraded to %s"], "1.0" ) )
	end
	if ( olddb.Options.General.dbVersion == "1.0" ) then
		olddb.Options.General.buttonTexture = nil
		olddb.Options.General.uiscale = 1
		olddb.Options.General.xPos = 100
		olddb.Options.General.yPos = -100
		olddb.firstLoad = nil
		olddb.Options.General.dbVersion = "1.1"
		addon.Print( string.format( L["Database upgraded to %s"], "1.1" ) )
	end
end

-- options stuff
local displayOptions = nil
local function giveDisplay()
	if (not displayOptions) then
		displayOptions = {
			order = 2,
			type = "group",
			name = L["Display Options"],
			desc = L["DISPLAY_OPTIONS_DESC"],
			args = {
				header1 = {
					order	= 21,
					type	= "header",
					name	= L["Display Options"],
				},
				desc = {
					order	= 22,
					type	= "description",
					name	= L["DISPLAY_OPTIONS_DESC"],
				},
				sound = {
					order	= 23,
					type	= "toggle",
					name	= L["Enable DEBUG"],
					desc	= L["DISPLAY_DEBUG_DESC"],
					get		= function() return db.options.display.debug end,
					set		= function() db.options.display.debug = not db.options.display.debug end,
				},
				uiscale = {
					order	= 24,
					type	= "range",
					name	= L["UI Scale"],
					desc	= L["UI_SCALE_DESC"],
					min		= .7,
					max		= 1.3,
					step	= .05,
					bigStep = .05,
					get		= function() return db.Options.General.uiscale end,
					set		= function( info, v ) 
								db.Options.General.uiscale = v
								-- TODO : Need a line here for setting the scale of the main parent frame
							  end,
				},
				autoopen = {
					order	= 25,
					type	= "toggle",
					name	= L["Toggle Auto-open"],
					desc	= L["TOGGLE_AUTOOPEN_DESC"],
					get		= function() return db.Options.General.autoopen end,
					set		= function() db.Options.General.autoopen = not db.Options.General.autoopen end,
				},
				classColors = {
					order	= 26,
					type	= "toggle",
					name	= L["Toggle Class Coloring"],
					desc	= L["TOGGLE_CLASSCOLOR_DESC"],
					get		= function() return db.Options.General.classColors end,
					set		= function() db.Options.General.classColors = not db.Options.General.classColors end,
				},
				raidonly = {
					order	= 27,
					type	= "toggle",
					name	= L["Toggle Raid Only View"],
					desc	= L["TOGGLE_RAIDONLY_DESC"],
					get		= function() return db.Options.General.raidonly end,
					set		= function() db.Options.General.raidonly = not db.Options.General.raidonly end,
				},
			},
		}
	end
	return displayOptions
end

local options = nil
local function fullOptions()
	if (not options) then
		options = {
			type = "group",
			name = MODNAME,
			args = {
				general = {
					order	= 1,
					type	= "group",
					name	= L["General Information"],
					desc	= L["GENERAL_INFO_DESC"],
					args	= {
						header1 = {
							order	= 11,
							type	= "header",
							name	= L["General Information"],
						},
						desc = {
							order	= 12,
							type	= "description",
							name	= L["GENERAL_INFO_DESC"],
						},
						author = {
							order	= 13,
							type	= "description",
							name	= L["Author : "] .. addonauthor .. "\n",
						},
						version = {
							order	= 14,
							type	= "description",
							name	= L["Version : "] .. addonversion .. "\n",
						},
						builddate = {
							order	= 15,
							type	= "description",
							name	= L["Build Date : "] .. builddate .. "\n",
						},
						dbversion = {
							order	= 15,
							type	= "description",
							name	= L["Database Version : "] .. db.options.display.dbVersion .. "\n",
						},
						translators = {
							order	= 16,
							type	= "description",
							name	= string.format( L["Helpful Translators (thank you) : %s"], " \n" )
						},
					},
				},
			},
		}
	end
	
	for k,v in pairs(modularOptions) do
		options.args[k] = (type(v) == "function") and v() or v
	end

	return options
end

function addon:SetupOptions()
	AceConfigReg:RegisterOptionsTable(MODNAME, fullOptions)
	self.optionsFrame = AceConfigDialog:AddToBlizOptions(MODNAME, nil, nil, "general")

	-- Fill up our modular options...
	self:RegisterModuleOptions("Display", giveDisplay(), L["Display Options"])
end

function addon:RegisterModuleOptions(name, optionsTable, displayName)
	modularOptions[name] = optionsTable
	self.optionsFrame[name] = AceConfigDialog:AddToBlizOptions(MODNAME, displayName, MODNAME, name)
end

function addon:ShowConfig()
	-- Open the profiles tab before, so the menu expands
	InterfaceOptionsFrame_OpenToCategory( self.optionsFrame["Display"] )
end


function addon:OnInitialize()
	-- Do Frame Creation stuff here
	-- addon:CreateFrames()
end

function addon:OnEnable()
    -- Called when the addon is loaded
	addon.me = UnitName( "player" )

	self.db = LibStub( "AceDB-3.0" ):New( MODNAME .. "_DB", nil, "Default" )
	-- do we have any data yet; for this toon? If not, init its table
		if not ( self.db.profile ) then
		sdbp = self.db.profile
		sdbp.Players = {}
		sdbp.Alts = {}
		sdbp.RollLists = {}
		sdbp.LastMod = {}
		sdbp.Log = {
			["SKG"] = {},
			["Loot"] = {},
		}
		sdbp.Snapshot = {}
		sdbp.SnapshotLM = {}
		sdbp.Options = {
			General = {
				debug = false,
				autoopen = false,
				classColors = true,
				raidonly = false,
				xPos = 100,
				yPos = -100,
				dbVersion = "1.1",
				uiscale = 1,
			},
			Security = {
				Policies = {},
			},
			Bid = {
				ArmorFilter = 0,
			},
			List = {},
		}
	else
		-- check if we need to upgrade the db
		addon.upgradeDB()
	end

	db = self.db.profile

	addon:SetupOptions()
	
	-- Event listeners
    -- addon:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
	addon:RegisterEvent( "ADDON_LOADED" )
	addon:RegisterEvent( "LOOT_OPENED" )
	addon:RegisterEvent( "RAID_ROSTER_UPDATE" )
	addon:RegisterEvent( "CHAT_MSG_WHISPER" )
	addon:RegisterEvent( "UNIT_SPELLCAST_SENT" )
	-- Other events to worry about sometime
	-- PARTY_LOOT_METHOD_CHANGED
	addon:RegisterEvent( "GUILD_ROSTER_UPDATE" )

	SKG_INIT_GUILD = 1

	-- Comm registration (handlers are in SKG_Comm.lua)
	addon:RegisterComm( MODNAME )

	-- Called when the addon is enabled
	local major, minor = addon.getVersionComponents()
	addon.Print( string.format( L["SKG_MSG_ADDON_LOADED"], major, minor ) )
end

function addon:OnDisable()
	-- do my best to make the addon disappear from view
	-- SKG_Frame:Hide()
	-- get rid of the chat command
	addon:UnregisterChatCommand( "skg" )
	addon:UnregisterAllEvents()
end

function addon:Disable()
	-- do my best to make the addon disappear from view
	-- SKG_Frame:Hide()
	-- get rid of the chat command
	addon:UnregisterChatCommand( "skg" )
	addon:UnregisterAllEvents()
end

-- Register our chat command, and set its function
addon:RegisterChatCommand("skg", "OnSlashCommand")

-- And now do the dirty work
function addon:OnSlashCommand( text )
	if ( text == "" ) then
		--[[
		if ( SKG_Frame:IsVisible() ) then
			PlaySound("igMainMenuClose");
			SKG_Frame:Hide()
		else
			PlaySound("igMainMenuOpen");
			SKG_Frame:Show()
		end
		]]--
	elseif ( text == "reset" ) then
		db.Options.General.xPos = 100
		db.Options.General.yPos = -100
		-- SKG_Frame:SetPoint( "TOPLEFT", UIParent, "TOPLEFT", 100, -100 )
		-- SKG_Frame:Show()
	elseif ( text == "checklists" ) then
		-- These commands should only work if you're a syncer (or above) 
		if ( SKG_SECURITY_LEVEL ~= "U" ) then
			addon.Print( L["SKG_MSG_DIAG"] )
			-- perform a diagnostic check on lists to see if they have corruption
			-- Check Players list
			--[[
			local rc = SKG_checkList( "Players" )
			SKG_Print( SKG_ListError( "Players", rc ) )
			-- Check Alts list
			rc = SKG_checkList( "Alts" )
			SKG_Print( SKG_ListError( "Alts", rc ) )
			-- loop through the rolling lists
			local db = SKG.db.profile.RollLists
			for k, v in pairs( db ) do
				rc = SKG_checkList( k )
				SKG_Print( SKG_ListError( k, rc ) )
			end
			]]--
			addon.Print( L["SKG_MSG_DIAGCOMPLETE"] )
		else
			addon.Print( L["SKG_ERROR_NOTLEET"] )
		end
	elseif ( text == "repairlists" ) then
		-- make an attempt to get rid of bad entries in lists
		if ( SKG_SECURITY_LEVEL ~= "U" ) then
			addon.Print( L["SKG_MSG_LISTREPAIR"] )
			-- SKG_RepairLists()
			addon.Print( L["SKG_MSG_REPAIRCOMPLETE"] )
		else
			addon.Print( L["SKG_ERROR_NOTLEET"] )
		end
	else
		addon.Print( L["SKG Usage:"] )
		addon.Print( "  " .. L["/skg - open SKG GUI"] )
		addon.Print( "  " .. L["/skg reset - reset SKG GUI position"] )
		addon.Print( "  " .. L["/skg checklists - check lists for corruption"] )
		addon.Print( "  " .. L["/skg repairlists - attempt to repair list corruption"] )
	end
end

--[[
************************************************************************
CHANGELOG:

Date : 1/25/09
	Rewrite
Date : 10/26/08
	Moving RegisterEvent calls to OnInit
	Moving RegisterComm call to OnInit
	Defined fubarOptionsTable to be global
Date : 10/21/08
	Initial attempt with Ace3 instead of LibRock. Probably will require
	major edits once. Added header/footer.
************************************************************************
]]--